/*
 * Copyright 2011 ssg.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package com.ssg.tools.jsonxml;

import com.ssg.tools.jsonxml.common.Utilities;
import com.ssg.tools.jsonxml.common.ValidationError;
import java.io.File;
import java.io.IOException;
import java.net.MalformedURLException;
import java.net.URI;
import java.net.URL;
import java.util.List;
import javax.xml.transform.stream.StreamSource;

/**
 * Provides URI/URL/File - based test of format compatibility
 * Provides basic validation of source. If (optional) schema - full validation, otherwise - syntax validity.
 * 
 * @author ssg
 */
public abstract class Tester {

    public abstract String getName();

    /**
     * Test if URL represents data compatible with tested format.
     * 
     * @param source
     * @return
     * @throws IOException 
     */
    public abstract boolean test(URL source) throws IOException;

    /**
     * Basic syntax validation and/or schema validation.
     * 
     * @param source
     * @param schema
     * @return list of found errors. If empty list - no errors were detected. NOT NULL!
     * @throws IOException 
     */
    public abstract List<ValidationError> validate(StreamSource source, StreamSource schema) throws IOException;

    public boolean test(String source) throws IOException {
        return test(Utilities.asURL(source));
    }

    public boolean test(URI source) throws IOException {
        try {
            return test(source.toURL());
        } catch (MalformedURLException muex) {
            return false;
        }
    }

    public boolean test(File source) throws IOException {
        return test(source.toURI());
    }
}
