/*
 * Copyright 2011 sesidoro.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 * 
 * Commonly used or basic classes for processors. Juat a mixture of commons.
 * - CRITICALITY - error criticality levels use in processors.
 * - DataValidation - in-memory validator (in contrast to source validation)
 *     used (may be not really needed) for generalization of JSON-schema based validation
 *     that is implemented on loaded object, not on source.
 * - Formats - converter of scalars (including date and string) to/from string representation.
 *     Formats provides variants of formatting and is adjustable to locale/time zone for defaults
 *     though allowing others in alterantive formats. 
 * - FormatterContext - base for "smnart" formatters that provides proper leading/trailing texts
 *     for various format elements (see STRUCTURE_FORMAT_ELEMENT for list).
 *     It extends StructureProcessingContext inheriting commonly usable stack, path, objects repository, 
 *     and reflection builder.
 * - ObjectsRegistry - provides a way of storing references to objects with path using System.identityHashCode()
 *     as keys. Introduced to prevent closed loops and override Java bug on recursive objects (e.g. if List contains as 
 *     item itself itsh hashCode() cannot be evaluated - stack overflow occurs).
 * - ParserContext - just to differ from complex FormatsContext though extends StructureProcessingContext.
 * - Path - path manipulation tool. Holds path as list of elements, including path and array separators.
 *     Provides simple interface for changing and checking path and converts it to a number of text formats.
 * - ReflectiveBuilder - set of related methods to simplify processing of generic java objects to and from text formats.
 *     Additionally provides Map wrapping/unwrapping to allow serialization on JSON incompatible data structures
 *     (maps with non-string keys).
 * - StructureErrorInfo - provides detailed info on occured error if processor could give it.
 * - StructureProcessingContext - combines commonly used features in various data processor types.
 *     These include:
 *     - stack
 *     - path
 *     - reflective builder
 *     - objects registry
 *     - others ...
 * - Utilities - collection of shared methods that do not fit into any other class.
 * - ValidationContext - base for creation of in-memory validators.
 * 
 */
package com.ssg.tools.jsonxml.common;
