/*
 * Copyright 2011 ssg.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package com.ssg.tools.jsonxml.json;

import com.ssg.tools.jsonxml.common.Formats;
import com.ssg.tools.jsonxml.common.ParserContext;
import java.util.Locale;

/**
 *
 * @author ssg
 */
public class JSONParserContext extends ParserContext {
    private boolean _strict = true;
    public static Object OBJECT_DELIMITER_TAG = new Object();
    public static final char OPEN_LIST = '[';
    public static final char OPEN_OBJECT = '{';
    public static final char CLOSE_LIST = ']';
    public static final char CLOSE_OBJECT = '}';
    public static final char OBJECTS_DELIMITER = ',';
    public static final char VALUE_DELIMITER = ':';
    private JSONFormatterContext _jsonFormat = new JSONFormatterContext();
    // parsing position. Line/Position are 1-based. GlobalPosition is 0-based stream posiotion.
    private int _line;
    private long _position;
    private long _globalPosition;
    private boolean _eof = false;

    public JSONParserContext() {
    }

    public JSONParserContext(ParserContext context) {
        if (context != null) {
            setStack(context.getStack());
            setPath(context.getPath());
            setFormats(context.getFormats());
        }
    }

    public JSONParserContext(Locale locale) {
    }

    /**
     * @return the _jsonFormat
     */
    public JSONFormatterContext getJsonFormat() {
        return _jsonFormat;
    }

    /**
     * @param jsonFormat the _jsonFormat to set
     */
    public void setJsonFormat(JSONFormatterContext jsonFormat) {
        this._jsonFormat = jsonFormat;
    }

    @Override
    public Formats getFormats() {
        if (_jsonFormat != null) {
            return _jsonFormat.getFormats();
        } else {
            return super.getFormats();
        }
    }

    @Override
    public void setFormats(Formats formats) {
        if (_jsonFormat != null) {
            _jsonFormat.setFormats(formats);
            _jsonFormat.enforceStrictSyntax();
        } else {
            super.setFormats(formats);
        }
    }

    /**
     * @return the _line
     */
    public int getLine() {
        return _line;
    }

    /**
     * @param line the _line to set
     */
    public void setLine(int line) {
        this._line = line;
    }

    /**
     * @return the _position
     */
    public long getPosition() {
        return _position;
    }

    /**
     * @param position the _position to set
     */
    public void setPosition(long position) {
        this._position = position;
    }

    /**
     * @return the _globalPosition
     */
    public long getGlobalPosition() {
        return _globalPosition;
    }

    /**
     * @param globalPosition the _globalPosition to set
     */
    public void setGlobalPosition(long globalPosition) {
        this._globalPosition = globalPosition;
    }

    /**
     * @return the _eof
     */
    public boolean isEOF() {
        return _eof;
    }

    /**
     * @param eof the _eof to set
     */
    public void setEOF(boolean eof) {
        this._eof = eof;
    }

    /**
     * @return the _strict
     */
    public boolean isStrict() {
        return _strict;
    }

    /**
     * @param strict the _strict to set
     */
    public void setStrict(boolean strict) {
        this._strict = strict;
    }
    
}
