/*
 * Copyright 2011 ssg.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package com.ssg.tools.jsonxml.json;

import com.ssg.tools.jsonxml.common.tools.BufferingReader;
import java.io.IOException;
import java.io.Reader;

/**
 * Structure used to keep parsing context.
 * It provides:
 *  - access to reader:
 *    * read()
 *    * unread()
 *  - next expected token type
 *  - reusable character array (to save time/memory)
 *  - currently evaluated object
 *
 * @author ssg
 */
public class JSONParserLiteContext {
    BufferingReader reader;
    Object object;
    JSONParserLite.NEXT next = JSONParserLite.NEXT.ITEM;
    boolean inObject = false;
    //StringBuilder sb = new StringBuilder();
    char[] string = new char[65535];

    public JSONParserLiteContext(Reader r) {
        this.reader = (r instanceof BufferingReader) ? (BufferingReader) r : new BufferingReader(r, 2048);
    }

    public Object getObject() {
        return object;
    }

    public int read() throws IOException {
        return reader.read();
    }

    public void unread() throws IOException {
        reader.unread();
    }
    
}
