/*
 * Copyright 2011 ssg.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package com.ssg.tools.jsonxml.json.schema;

import java.util.EnumMap;
import java.util.Map;

/**
 *
 * @author ssg
 */
public class JBase implements Cloneable {

    Map<ATTRIBUTE, Object> _attributes = new EnumMap<ATTRIBUTE, Object>(ATTRIBUTE.class);

    protected void set(ATTRIBUTE attr, Object value) {
        if (value != null) {
            _attributes.put(attr, value);
        } else if (_attributes.containsKey(attr)) {
            _attributes.remove(attr);
        }
    }

    protected boolean exists(ATTRIBUTE attr) {
        return _attributes.containsKey(attr);
    }

    public boolean emptySchema() {
        return _attributes.isEmpty();
    }

    @Override
    public boolean equals(Object obj) {
        if (obj == null) {
            return false;
        }
        if (getClass() != obj.getClass()) {
            return false;
        }
        final JBase other = (JBase) obj;
        if (this._attributes != other._attributes && (this._attributes == null || !this._attributes.equals(other._attributes))) {
            return false;
        }
        return true;
    }

    @Override
    public int hashCode() {
        int hash = 7;
        hash = 47 * hash + (this._attributes != null ? this._attributes.hashCode() : 0);
        return hash;
    }
}
