/*
 * Copyright 2011 ssg.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package com.ssg.tools.jsonxml.json.schema;

import java.security.InvalidParameterException;

/**
 * Encapsulates predefined type (string value) or JSON schema as type definition.
 *
 * @author ssg
 */
public class JSONValueExtends extends JSONMixedAttribute {

    public static final String __SCHEMA = "http://json-schema.org/schema";
    public static final String __HYPER_SCHEMA = "http://json-schema.org/hyper-schema";
    public static final String[] PREDEFINED = {__SCHEMA, __HYPER_SCHEMA};
    public static final JSONValueExtends SCHEMA = new JSONValueExtends(__SCHEMA);
    public static final JSONValueExtends HYPER_SCHEMA = new JSONValueExtends(__HYPER_SCHEMA);

    public JSONValueExtends(String value) {
        if (validate(value)) {
            _value = value;
        } else {
            // any6 value is OK
            //throw new InvalidParameterException("Invalid type: " + value + ". Need schema or one of predefined types: " + PREDEFINED);
        }
    }

    public JSONValueExtends(JSchema schema) throws JSONSchemaException {
        _value = schema;
        if (schema == null) {
            throw new JSONSchemaException("Invalid schema for type: null. Need schema or one of predefined types: " + PREDEFINED);
        }
    }

    @Override
    public boolean validate(Object value) {
        if (value instanceof JSchema) {
            return true;
        }
        if (value instanceof String) {
            for (String v : PREDEFINED) {
                if (v.equals((String) value)) {
                    return true;
                }
            }
        }
        return false;
    }
}
