/*
 * Copyright 2011 ssg.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package com.ssg.tools.jsonxml.json.schema;

import java.text.DateFormat;
import java.text.SimpleDateFormat;

/**
 *
 * @author ssg
 */
public class JSONValueFormat extends JSONMixedAttribute {

    public static final DateFormat F_DATE_TIME = new SimpleDateFormat("yyyy-MM-dd'T'hh:mm:ssZ");
    public static final DateFormat F_DATE = new SimpleDateFormat("yyyy-MM-dd");
    public static final DateFormat F_TIME = new SimpleDateFormat("hh:mm:ss");

    /**
     * List of predefined color names as specified in http://www.w3.org/TR/2007/CR-CSS21-20070719
     */
    public enum F_COLOR_NAME {

        aqua, black, blue, fuchsia, gray, green,
        lime, maroon, navy, olive, orange, purple,
        red, silver, teal, white, yellow
    }
    public static final String __DATE_TIME = "date-time";
    public static final String __DATE = "date";
    public static final String __TIME = "time";
    public static final String __UTC_MILLISEC = "utc-millisec";
    public static final String __REGEX = "regex";
    public static final String __COLOR = "color";
    public static final String __STYLE = "style";
    public static final String __PHONE = "phone";
    public static final String __URI = "uri";
    public static final String __EMAIL = "email";
    public static final String __IP_ADDRESS = "ip-address";
    public static final String __IPV6 = "ipv6";
    public static final String __HOST_NAME = "host-name";
    public static final String[] PREDEFINED = {
        __DATE_TIME,
        __DATE,
        __TIME,
        __UTC_MILLISEC,
        __REGEX,
        __COLOR,
        __STYLE,
        __PHONE,
        __URI,
        __EMAIL,
        __IP_ADDRESS,
        __IPV6,
        __HOST_NAME};
    public static final JSONValueFormat DATE_TIME = new JSONValueFormat(__DATE_TIME);
    public static final JSONValueFormat DATE = new JSONValueFormat(__DATE);
    public static final JSONValueFormat TIME = new JSONValueFormat(__TIME);
    public static final JSONValueFormat UTC_MILLISEC = new JSONValueFormat(__UTC_MILLISEC);
    public static final JSONValueFormat REGEX = new JSONValueFormat(__REGEX);
    public static final JSONValueFormat COLOR = new JSONValueFormat(__COLOR);
    public static final JSONValueFormat STYLE = new JSONValueFormat(__STYLE);
    public static final JSONValueFormat PHONE = new JSONValueFormat(__PHONE);
    public static final JSONValueFormat URI = new JSONValueFormat(__URI);
    public static final JSONValueFormat EMAIL = new JSONValueFormat(__EMAIL);
    public static final JSONValueFormat IP_ADDRESS = new JSONValueFormat(__IP_ADDRESS);
    public static final JSONValueFormat IPV6 = new JSONValueFormat(__IPV6);
    public static final JSONValueFormat HOST_NAME = new JSONValueFormat(__HOST_NAME);

    public JSONValueFormat(String value) {
        if (validate(value)) {
            _value = value;
        } else {
            //throw new InvalidParameterException("Invalid format: " + value + ". Need schema or one of predefined formate: " + PREDEFINED);
            _value = value;
            _custom = true;
        }
    }

    public JSONValueFormat(JSchema schema) throws JSONSchemaException {
        _value = schema;
        if (schema == null) {
            throw new JSONSchemaException("Invalid schema for format: null. Need schema or one of predefined formats: " + PREDEFINED);
        }
    }

    @Override
    public boolean validate(Object value) {
        if (value instanceof JSchema) {
            return true;
        }
        if (value instanceof String) {
            for (String v : PREDEFINED) {
                if (v.equals((String) value)) {
                    return true;
                }
            }
        }
        return false;
    }
}
