/*
 * Copyright 2011 ssg.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package com.ssg.tools.jsonxml.json.schema;

import java.security.InvalidParameterException;

/**
 * Encapsulates predefined type (string value) or JSON schema as type definition.
 *
 * @author ssg
 */
public class JSONValueType extends JSONMixedAttribute {

    public static final String __STRING = "string";
    public static final String __NUMBER = "number";
    public static final String __INTEGER = "integer";
    public static final String __BOOLEAN = "boolean";
    public static final String __OBJECT = "object";
    public static final String __ARRAY = "array";
    public static final String __NULL = "null";
    public static final String __ANY = "any";
    public static final String[] PREDEFINED = {__STRING, __NUMBER, __INTEGER, __BOOLEAN, __OBJECT, __ARRAY, __NULL, __ANY};
    public static final JSONValueType STRING = new JSONValueType(__STRING);
    public static final JSONValueType NUMBER = new JSONValueType(__NUMBER);
    public static final JSONValueType INTEGER = new JSONValueType(__INTEGER);
    public static final JSONValueType BOOLEAN = new JSONValueType(__BOOLEAN);
    public static final JSONValueType OBJECT = new JSONValueType(__OBJECT);
    public static final JSONValueType ARRAY = new JSONValueType(__ARRAY);
    public static final JSONValueType NULL = new JSONValueType(__NULL);
    public static final JSONValueType ANY = new JSONValueType(__ANY);
    public static final JSONValueType[] PREDEFINED_TYPES = {STRING, NUMBER, INTEGER, BOOLEAN, OBJECT, ARRAY, NULL, ANY};

    public JSONValueType(String value) {
        if (validate(value)) {
            _value = value;
        } else {
            throw new InvalidParameterException("Invalid type: " + value + ". Need schema or one of predefined types: " + PREDEFINED);
        }
    }

    public JSONValueType(JSchema schema) throws JSONSchemaException {
        _value = schema;
        if (schema == null) {
            throw new JSONSchemaException("Invalid schema for type: null. Need schema or one of predefined types: " + PREDEFINED);
        }
    }

    @Override
    public boolean validate(Object value) {
        if (value instanceof JSchema) {
            return true;
        }
        if (value instanceof String) {
            for (String v : PREDEFINED) {
                if (v.equals((String) value)) {
                    return true;
                }
            }
        }
        return false;
    }

}
