/*
 * Copyright 2011 ssg.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package com.ssg.tools.jsonxml.json.schema;

import com.ssg.tools.jsonxml.common.Path;
import com.ssg.tools.jsonxml.common.Utilities;
import com.ssg.tools.jsonxml.common.tools.NonResizableListForArray;
import java.lang.reflect.Method;
import java.net.URI;
import java.nio.charset.Charset;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.Collection;
import java.util.Collections;
import java.util.Enumeration;
import java.util.HashSet;
import java.util.Iterator;
import java.util.LinkedHashMap;
import java.util.List;
import java.util.Map;
import java.util.Set;
import java.util.regex.Pattern;
import java.util.regex.PatternSyntaxException;

/**
 *
 * @author ssg
 */
public class JSchema extends JBase {

    String _name;

    public JSchema() {
    }

    public String getName() {
        return _name;
    }

    public void setName(String name) {
        _name = name;
    }

    /**
     * TODO: types list may include JSchema instances... !!!
     * @return
     * @throws JSONSchemaException 
     */
    public List<JSONValueType> getType() throws JSONSchemaException {
        Object o = _attributes.get(ATTRIBUTE._type);
        if (o == null) {
            return null;
        } else {
            return (List<JSONValueType>) o;
        }
    }

    public void setType(JSONValueType type) throws JSONSchemaException {
        set(ATTRIBUTE._type, (type != null) ? Collections.singletonList(type) : null);
    }

    public void setType(JSONValueType[] types) throws JSONSchemaException {
        set(ATTRIBUTE._type, (types != null) ? Arrays.asList(types) : null);
    }

    private void setType(List<JSONValueType> types) throws JSONSchemaException {
        set(ATTRIBUTE._type, types);
    }

    public Map<String, JSchema> getProperties() throws JSONSchemaException {
        return (Map<String, JSchema>) _attributes.get(ATTRIBUTE._properties);
    }

    public JSchema getProperty(String name) throws JSONSchemaException {
        Map<String, JSchema> props = getProperties();
        if (props != null) {
            return props.get(name);
        } else {
            return null;
        }
    }

    public boolean hasProperties() throws JSONSchemaException {
        Map<String, JSchema> ps = getProperties();
        return ps != null && !ps.isEmpty();
    }

    public void setProperties(Map<String, JSchema> properties) throws JSONSchemaException {
        set(ATTRIBUTE._properties, properties);
    }

    public void setProperty(String name, JSchema value) throws JSONSchemaException {
        Map<String, JSchema> props = getProperties();
        if (props == null) {
            props = new LinkedHashMap<String, JSchema>();
            set(ATTRIBUTE._properties, props);
        }
        props.put(name, value);
    }

    public Map<String, JSchema> getPatternProperties() throws JSONSchemaException {
        return (Map<String, JSchema>) _attributes.get(ATTRIBUTE._patternProperties);
    }

    public JSchema getPatternProperty(String name) throws JSONSchemaException {
        if (name == null) {
            return null;
        }
        Map<String, JSchema> props = getPatternProperties();
        if (props != null) {
            return props.get(name);
        } else {
            return null;
        }
    }

    public void setPatternProperties(Map<String, JSchema> properties) throws JSONSchemaException {
        set(ATTRIBUTE._patternProperties, properties);
    }

    public void setPatternProperty(String name, JSchema value) throws JSONSchemaException {
        Map<String, JSchema> props = getPatternProperties();
        if (props == null) {
            props = new LinkedHashMap<String, JSchema>();
            set(ATTRIBUTE._patternProperties, props);
        }
        props.put(name, value);
    }

    public JSchema getAdditionalProperties() throws JSONSchemaException {
        Object o = _attributes.get(ATTRIBUTE._additionalProperties);
        if (o instanceof JSchema) {
            return (JSchema) o;
        } else {
            return null;
        }
    }

    public Boolean isAdditionalProperties() throws JSONSchemaException {
        Object o = _attributes.get(ATTRIBUTE._additionalProperties);
        if (o instanceof Boolean) {
            return (Boolean) o;
        } else {
            return true;
        }
    }

    public void setAdditionalProperties(JSchema properties) throws JSONSchemaException {
        set(ATTRIBUTE._additionalProperties, properties);
    }

    public void setAdditionalProperties(Boolean allowed) throws JSONSchemaException {
        set(ATTRIBUTE._additionalProperties, allowed);
    }

    public JSchema getItems() throws JSONSchemaException {
        return (JSchema) _attributes.get(ATTRIBUTE._items);
    }

    public void setItems(JSchema value) throws JSONSchemaException {
        set(ATTRIBUTE._items, value);
    }

    public JSchema getAdditionalItems() throws JSONSchemaException {
        Object o = _attributes.get(ATTRIBUTE._additionalItems);
        if (o instanceof JSchema) {
            return (JSchema) o;
        } else {
            return null;
        }
    }

    public Boolean isAdditionalItems() throws JSONSchemaException {
        Object o = _attributes.get(ATTRIBUTE._additionalItems);
        if (o instanceof Boolean) {
            return (Boolean) o;
        } else {
            return true;
        }
    }

    public void setAdditionalItems(JSchema value) throws JSONSchemaException {
        set(ATTRIBUTE._additionalItems, value);
    }

    public void setAdditionalItems(Boolean value) throws JSONSchemaException {
        set(ATTRIBUTE._additionalItems, value);
    }

    public Boolean isRequired() throws JSONSchemaException {
        Boolean b = (Boolean) _attributes.get(ATTRIBUTE._required);
        return b != null && b;
    }

    public void setRequired(Boolean value) throws JSONSchemaException {
        set(ATTRIBUTE._required, value);
    }

    public Map<String, Object> getDependencies() throws JSONSchemaException {
        return (Map<String, Object>) _attributes.get(ATTRIBUTE._dependencies);
    }

    public void setDependencies(Map<String, Object> value) throws JSONSchemaException {
        set(ATTRIBUTE._dependencies, value);
    }

    public Number getMinimum() throws JSONSchemaException {
        return (Number) _attributes.get(ATTRIBUTE._minimum);
    }

    public void setMinimum(Number value) throws JSONSchemaException {
        set(ATTRIBUTE._minimum, value);
    }

    public Number getMaximum() throws JSONSchemaException {
        return (Number) _attributes.get(ATTRIBUTE._maximum);
    }

    public void setMaximum(Number value) throws JSONSchemaException {
        set(ATTRIBUTE._maximum, value);
    }

    public Boolean isExclusiveMinimum() throws JSONSchemaException {
        Boolean b = (Boolean) _attributes.get(ATTRIBUTE._exclusiveMinimum);
        return b != null && b;
    }

    public void setExclusiveMinimum(Boolean value) throws JSONSchemaException {
        set(ATTRIBUTE._exclusiveMinimum, value);
    }

    public Boolean isExclusiveMaximum() throws JSONSchemaException {
        Boolean b = (Boolean) _attributes.get(ATTRIBUTE._exclusiveMaximum);
        return b != null && b;
    }

    public void setExclusiveMaximum(Boolean value) throws JSONSchemaException {
        set(ATTRIBUTE._exclusiveMaximum, value);
    }

    public Integer getMinItems() throws JSONSchemaException {
        return (Integer) _attributes.get(ATTRIBUTE._minItems);
    }

    public void setMinItems(Integer value) throws JSONSchemaException {
        if (value != null && value < 0) {
            throw new JSONSchemaException("Minimum number of list items cannot be negative: " + value);
        }
        set(ATTRIBUTE._minItems, value);
    }

    public Integer getMaxItems() throws JSONSchemaException {
        return (Integer) _attributes.get(ATTRIBUTE._maxItems);
    }

    public void setMaxItems(Integer value) throws JSONSchemaException {
        if (value != null && value < 0) {
            throw new JSONSchemaException("Maximum number of list items cannot be negative: " + value);
        }
        set(ATTRIBUTE._maxItems, value);
    }

    public Boolean isUniqueItems() throws JSONSchemaException {
        Boolean b = (Boolean) _attributes.get(ATTRIBUTE._uniqueItems);
        return b != null && b;
    }

    public void setUniqueItems(Boolean value) throws JSONSchemaException {
        set(ATTRIBUTE._uniqueItems, value);
    }

    public String getPattern() throws JSONSchemaException {
        return (String) _attributes.get(ATTRIBUTE._pattern);
    }

    public void setPattern(String value) throws JSONSchemaException {
        if (value != null) {
            try {
                Pattern p = Pattern.compile(value);
            } catch (PatternSyntaxException psex) {
                throw new JSONSchemaException("Invalid regular expression for pattern: " + value + ". ERROR: " + psex, psex);
            }
        }
        set(ATTRIBUTE._pattern, value);
    }

    public Integer getMinLength() throws JSONSchemaException {
        return (Integer) _attributes.get(ATTRIBUTE._minLength);
    }

    public void setMinLength(Integer value) throws JSONSchemaException {
        if (value != null && value < 0) {
            throw new JSONSchemaException("Minimal length cannot be negative: " + value);
        }
        set(ATTRIBUTE._minLength, value);
    }

    public Integer getMaxLength() throws JSONSchemaException {
        return (Integer) _attributes.get(ATTRIBUTE._maxLength);
    }

    public void setMaxLength(Integer value) throws JSONSchemaException {
        if (value != null && value < 0) {
            throw new JSONSchemaException("Minimal length cannot be negative: " + value);
        }
        set(ATTRIBUTE._maxLength, value);
    }

    public List getEnum() throws JSONSchemaException {
        return (List) _attributes.get(ATTRIBUTE._enum);
    }

    public void setEnum(List values) throws JSONSchemaException {
        if (values != null) {
            Set set = new HashSet();
            for (Object value : values) {
                if (set.contains(value)) {
                    throw new JSONSchemaException("Enumeration must contain only UNIQUE items. Duplicate item(s) found: " + value);
                }
                set.add(value);
            }
        }
        set(ATTRIBUTE._enum, values);
    }

    public void setEnum(Object[] values) throws JSONSchemaException {
        if (values != null) {
            Set set = new HashSet();
            for (Object value : values) {
                if (set.contains(value)) {
                    throw new JSONSchemaException("Enumeration must contain only UNIQUE items. Duplicate item(s) found: " + value);
                }
                set.add(value);
            }
        }
        if (values != null) {
            setEnum(Arrays.asList(values));
        } else {
            setEnum((List) null);
        }
    }

    /**
     * Generic enumeration evaluator:
     * can build enum from array, list, collection, iterable, iterator, enumeration or enum class.
     * @param values
     * @throws JSONSchemaException 
     */
    public void setEnum(Object values) throws JSONSchemaException {
        if (values == null) {
            setEnum((List) null);
        } else if (values.getClass().isArray()) {
            setEnum(new NonResizableListForArray(values));
        } else if (values instanceof List) {
            setEnum((List) values);
        } else if (values instanceof Collection) {
            List evs = new ArrayList();
            Iterator it = ((Collection) values).iterator();
            while (it.hasNext()) {
                evs.add(it.next());
            }
            setEnum(evs);
        } else if (values instanceof Iterable) {
            List evs = new ArrayList();
            for (Object o : ((Iterable) values)) {
                evs.add(o);
            }
            setEnum(evs);
        } else if (values instanceof Iterator) {
            List evs = new ArrayList();
            while (((Iterator) values).hasNext()) {
                evs.add(((Iterator) values).next());
            }
            setEnum(evs);
        } else if (values instanceof Enumeration) {
            setEnum(Collections.list((Enumeration) values));
        } else if (values instanceof Class && ((Class) values).isEnum()) {
            try {
                Method m = ((Class) values).getMethod("values");
                Object[] evs = (Object[]) m.invoke(values);
                String[] sevs = new String[evs.length];
                for (int i = 0; i < evs.length; i++) {
                    sevs[i] = "" + evs[i];
                }
                setEnum(sevs);
                setEnum(evs);
            } catch (Throwable th) {
                throw new JSONSchemaException("Failed to build schema enumeration from enumeration class: " + values);
            }
        } else {
            setEnum(Collections.singletonList(values));
        }
    }

    public Object getDefault() throws JSONSchemaException {
        Object prototype = _attributes.get(ATTRIBUTE._default);
        if (prototype == null) {
            return null;
        }
        if (Utilities.isScalarOrString(prototype)) {
            return prototype;
        }
        if (prototype instanceof Map) {
            Map map = new LinkedHashMap();
            for (Object key : ((Map) prototype).keySet()) {
                map.put(key, ((Map) prototype).get(key));
            }
            return map;
        } else if (prototype instanceof List) {
            List list = new ArrayList();
            for (Object obj : ((List) prototype)) {
                list.add(obj);
            }
            return list;
        } else {
            try {
                Method m = prototype.getClass().getDeclaredMethod("clone");
                prototype = m.invoke(prototype);
                return prototype;
            } catch (Throwable th) {
                throw new JSONSchemaException("Failed to clone non-primitive default " + prototype + ": " + th, th);
            }
        }
    }

    public boolean hasDefault() {
        return exists(ATTRIBUTE._default);
    }

    /**
     * We can use as default only immutable or cloneable items.
     * @param value
     * @throws JSONSchemaException 
     */
    public void setDefault(Object value) throws JSONSchemaException {
        if (value != null) {
            if (Utilities.isScalarOrString(value)) {
                // no problem, will return the value
            } else {
                if (value instanceof Map || value instanceof List) {
                    // we'll create default map/list and copy values there...
                } else if (value instanceof Cloneable) {
                    try {
                        Method m = value.getClass().getDeclaredMethod("clone");
                        value = m.invoke(value);
                    } catch (Throwable th) {
                        throw new JSONSchemaException("Failed to clone non-primitive default " + value + ": " + th, th);
                    }
                } else {
                    throw new JSONSchemaException("Default MUST be cloneable or immutable object.");
                }
            }
        }
        set(ATTRIBUTE._default, value);
    }

    public String getTitle() throws JSONSchemaException {
        return (String) _attributes.get(ATTRIBUTE._title);
    }

    public void setTitle(String value) throws JSONSchemaException {
        set(ATTRIBUTE._title, value);
    }

    public String getDescription() throws JSONSchemaException {
        return (String) _attributes.get(ATTRIBUTE._description);
    }

    public void setDescription(String value) throws JSONSchemaException {
        set(ATTRIBUTE._description, value);
    }

    public JSONValueFormat getFormat() throws JSONSchemaException {
        return (JSONValueFormat) _attributes.get(ATTRIBUTE._format);
    }

    public void setFormat(JSONValueFormat value) throws JSONSchemaException {
        set(ATTRIBUTE._format, value);
    }

    public Number getDivisibleBy() throws JSONSchemaException {
        return (Number) _attributes.get(ATTRIBUTE._divisibleBy);
    }

    public void setDivisibleBy(Number value) throws JSONSchemaException {
        set(ATTRIBUTE._divisibleBy, value);
    }

    public List<JSONValueType> getDisallow() throws JSONSchemaException {
        Object o = _attributes.get(ATTRIBUTE._disallow);
        if (o == null) {
            return null;
        } else {
            return (List<JSONValueType>) o;
        }
    }

    public void setDisallow(JSONValueType type) throws JSONSchemaException {
        set(ATTRIBUTE._disallow, (type != null) ? Collections.singletonList(type) : null);
    }

    public void setDisallow(JSONValueType[] types) throws JSONSchemaException {
        set(ATTRIBUTE._disallow, (types != null) ? Arrays.asList(types) : null);
    }

    private void setDisallow(List<JSONValueType> types) throws JSONSchemaException {
        set(ATTRIBUTE._disallow, types);
    }

    public List<JSONValueExtends> getExtends() throws JSONSchemaException {
        Object o = _attributes.get(ATTRIBUTE._extends);
        if (o == null) {
            return null;
        } else if (o instanceof List) {
            return (List<JSONValueExtends>) o;
        } else if (o.getClass().isArray()) {
            return (List<JSONValueExtends>) Arrays.asList((JSONValueExtends[]) o);
        } else if (o instanceof JSONValueExtends) {
            return Collections.singletonList((JSONValueExtends) o);
        } else {
            throw new JSONSchemaException("Illegal type definition: " + o);
        }
    }

    public void setExtends(JSONValueExtends extend) throws JSONSchemaException {
        set(ATTRIBUTE._extends, extend);
    }

    public void setExtends(JSONValueExtends[] extend) throws JSONSchemaException {
        set(ATTRIBUTE._extends, (extend != null) ? Arrays.asList(extend) : null);
    }

    private void setExtends(List<JSONValueExtends> extend) throws JSONSchemaException {
        set(ATTRIBUTE._extends, extend);
    }

    public URI getId() throws JSONSchemaException {
        return (URI) _attributes.get(ATTRIBUTE._id);
    }

    public void setId(URI value) throws JSONSchemaException {
        set(ATTRIBUTE._id, value);
    }

    public URI get$ref() throws JSONSchemaException {
        return (URI) _attributes.get(ATTRIBUTE.$ref);
    }

    public void set$ref(URI value) throws JSONSchemaException {
        set(ATTRIBUTE.$ref, value);
    }

    public URI get$schema() throws JSONSchemaException {
        return (URI) _attributes.get(ATTRIBUTE.$schema);
    }

    public void set$schema(URI value) throws JSONSchemaException {
        set(ATTRIBUTE.$schema, value);
    }

    //////////////////////////// hyper
    public List<JLink> getLinks() {
        return (List<JLink>) _attributes.get(ATTRIBUTE._links);
    }

    public void setLinks(List<JLink> links) throws JSONSchemaException {
        set(ATTRIBUTE._links, links);
    }

    public void setLinks(JLink[] links) throws JSONSchemaException {
        set(ATTRIBUTE._links, (links != null) ? Arrays.asList(links) : null);
    }

    public FRAGMENT_PROTOCOL getFragmentResolution() {
        return (FRAGMENT_PROTOCOL) _attributes.get(ATTRIBUTE._fragmentResolution);
    }

    public void setFragmentResolution(FRAGMENT_PROTOCOL value) throws JSONSchemaException {
        set(ATTRIBUTE._fragmentResolution, value);
    }

    public Boolean isReadonly() {
        if (exists(ATTRIBUTE._readonly)) {
            return (Boolean) _attributes.get(ATTRIBUTE._readonly);
        } else {
            return false;
        }
    }

    public void setReadonly(Boolean value) {
        set(ATTRIBUTE._readonly, value);
    }

    public String getContentEncoding() {
        return (String) _attributes.get(ATTRIBUTE._contentEncoding);
    }

    public void setContentEncoding(String value) throws JSONSchemaException {
        if (value != null && !Charset.isSupported(value)) {
            throw new JSONSchemaException("Unsupported content encoding: " + value);
        }
        set(ATTRIBUTE._contentEncoding, value);
    }

    public URI getPathStart() throws JSONSchemaException {
        return (URI) _attributes.get(ATTRIBUTE._pathStart);
    }

    public void setPathStart(URI value) throws JSONSchemaException {
        set(ATTRIBUTE._pathStart, value);
    }

    public String getMediaType() {
        return (String) _attributes.get(ATTRIBUTE._mediaType);
    }

    public void setMediaType(String value) throws JSONSchemaException {
        set(ATTRIBUTE._mediaType, value);
    }

    /**
     * Validation is performed using following algorithm:
     * 1. validate using referenced schema (if any and available).
     * 2. validate using all available attribute validators.
     * 3. validate using appropriate validator for data (e.g. as Array or Object)
     * 
     * @param ctx
     * @param data
     * @param recursive
     * @return
     * @throws JSONSchemaException 
     */
    public boolean validate(JSONValidationContext ctx, Object data, boolean recursive) throws JSONSchemaException {
        boolean result = true;
        JSchema depSchema = ctx.getDependency(this);
        if (depSchema != null) {
            result = depSchema.validate(ctx, data, recursive);
        }
        Path path = ctx.getPath();
        //System.out.println("Validating " + path);
        for (ATTRIBUTE attr : ATTRIBUTE.values()) {
            JSONAttributeValidator av = ctx.getValidator(attr);
            if (av != null) {
                if (!av.validate(ctx, data, this)) {
                    result = false;
                }
            } else {
                // TODO: no validator for attribute... Do we need info on it?!
            }
        }
        if (data != null) {
            JSONDataValidator v = ctx.getValidator(data);
            if (v != null) {
                if (!v.validate(ctx, data, this, recursive)) {
                    result = false;
                }
            } else {
                // TODO: no validator for object type: do we need any?!
            }
        }
        return result;
    }

    @Override
    public boolean equals(Object obj) {
        if (obj == null) {
            return false;
        }
        if (getClass() != obj.getClass()) {
            return false;
        }
        final JSchema other = (JSchema) obj;
        if ((this._name == null) ? (other._name != null) : !this._name.equals(other._name)) {
            return false;
        }
        return super.equals(obj);
    }

    @Override
    public int hashCode() {
        int hash = 7;
        hash = 79 * hash + (this._name != null ? this._name.hashCode() : 0);
        return hash + super.hashCode();
    }
}
