/*
 * Copyright 2011 sesidoro.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 * 
 * 
 * Implementation of JSON schema as defined at http://tools.ietf.org/html/draft-zyp-json-schema-03.
 * Main schema element is implemented in class JSchema as set of accessors.
 * Actual storage mechanism of schema attributes/properties is implemented in JBase class.
 * Hyper-schema, mentioned in specification, is implemented in derived JLink class.
 * 
 * All schema attributes are internally accessed using enumeration ATTRIBUTE.
 * Other enumerations are defined for predefined value sets only.
 * 
 * Some schema attributes may have scalar or sub-schema(s) realization. 
 * These are absttracted via JSONMixedAttribute base class and implemented for following attributes:
 * - extends
 * - format
 * - type / disallow
 * 
 * For loading from/storing to JSON format ReflectiveBuilder extension is used, class JSONSchemaReflectiveBuilder.
 * Its role is proper adjustment of JSON schema names to Java classes. It is used to build JSchema-s from 
 * list/map/property Java objects structure and vice versa.
 * 
 * Actual operations with schema are executed via JSONSchema class, that realizes
 * ValidationSchema and provides recursive schema loading, validation, and storing methods.
 * 
 * Other classes implement support functionality for JSONSchema though may be used separately:
 * - JSONDataGenerator - sample tool for generation of sample data based on schema.
 * - JSONSchemaBuilder - scans data and builds reference schema on it (JSONSchema),
 *     Resultant schema may be stored to file and edited to reflect actually needed rules.
 * - JSONValidationContext - provides context for processing validation rules using schema.
 *     Context provides access to schema validators, shcema loader, and other utility functions.
 *     Also context may be used to initiate validation.
 * - JSONDataValidator - JSON schema-specific extension of DataValidator interface.
 *    Provides method to validate and provides info on attributes tested by this validator.
 * - JSONAttributeValidator - attribute-specific validator.
 * - JSONValidatorDefaultImpl - set of default implementations of attribute and other validators
 *    that are registered in default JSONValidationContext.
 * 
 * 
 */
package com.ssg.tools.jsonxml.json.schema;
