/*
 * Copyright 2011 ssg.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package com.ssg.tools.jsonxml.xml;

import com.ssg.tools.jsonxml.Parser;
import com.ssg.tools.jsonxml.Tester;
import com.ssg.tools.jsonxml.common.CRITICALITY;
import com.ssg.tools.jsonxml.common.ValidationError;
import java.io.BufferedInputStream;
import java.io.IOException;
import java.io.InputStream;
import java.io.InputStreamReader;
import java.net.URL;
import java.util.Collections;
import java.util.List;
import javax.xml.XMLConstants;
import javax.xml.transform.stream.StreamSource;
import javax.xml.validation.Schema;
import javax.xml.validation.SchemaFactory;
import javax.xml.validation.Validator;
import org.xml.sax.SAXException;

/**
 *
 * @author ssg
 */
public class TesterXML extends Tester {

    @Override
    public boolean test(URL source) throws IOException {
        InputStream is = null;
        try {
            is = new BufferedInputStream(source.openStream(), 256);
            int c = 0;
            while ((c = is.read()) != Parser.READER_EOF) {
                if (Parser.isWhiteSpace((char) c)) {
                    continue;
                }
                if (c == '<') {
                    return true;
                } else {
                    return false;
                }
            }
        } catch (IOException ioex) {
        } finally {
            try {
                if (is != null) {
                    is.close();
                }
            } catch (IOException ioex1) {
            }
        }
        return false;
    }

    @Override
    public String getName() {
        return "XML";
    }

    @Override
    public List<ValidationError> validate(StreamSource xml, StreamSource xsd) throws IOException {
        if (xsd == null) {
            try {
                XMLParser.test(xml.getReader() != null ? xml.getReader() : new InputStreamReader(xml.getInputStream(), "UTF-8"));
                return Collections.EMPTY_LIST;
            } catch (SAXException sex) {
                final String err=""+sex;
                return Collections.singletonList((ValidationError) new ValidationError(CRITICALITY.error) {

                    @Override
                    public String getMessage() {
                        return "XML validation failed: "+err;
                    }
                });
            } catch (IOException ioex) {
                final String err=""+ioex;
                return Collections.singletonList((ValidationError) new ValidationError(CRITICALITY.error) {

                    @Override
                    public String getMessage() {
                        return "XML validation failed: "+err;
                    }
                });
            }
        } else {
            try {
                SchemaFactory factory = SchemaFactory.newInstance(XMLConstants.W3C_XML_SCHEMA_NS_URI);
                Schema schema = factory.newSchema(xsd);
                Validator validator = schema.newValidator();
                validator.validate(xml);
                return Collections.EMPTY_LIST;
            } catch (SAXException sex) {
                final String err = "" + sex;
                return Collections.singletonList((ValidationError) new ValidationError(CRITICALITY.error) {

                    @Override
                    public String getMessage() {
                        return "XML validation failed: " + err;
                    }
                });
            }
        }
    }
}
